#!/usr/bin/env python3

from pyb00st.movehub import MoveHub
from pyb00st.constants import *
from inputs import get_gamepad
from espeak import espeak

from time import sleep
import threading

# Boost MAC address
MY_MOVEHUB_ADD = '00:16:53:A1:62:24'

# language
espeak.set_voice = 'en'

# global variables
emergency_stop = False
dangersaid = False
workingloop = True
colorsaid = False

# start direction
direction = "forward"

# speed
speed = 40

sleep(1)

# MotorThread is always running
# motors are controlled by changing right/left duty cycles
class MotorsThread(threading.Thread):

    right_dc = 0
    left_dc = 0
    head_dc = 0

    def __init__(self):
        self.running = True
        threading.Thread.__init__(self)
        print("Motor Thread Ready")

    def run(self):
        global emergency_stop

        while self.running:
            if not emergency_stop:
                if self.left_dc != 0:
                    mymovehub.run_motor_for_time(MOTOR_A, 300, self.left_dc)
                if self.right_dc != 0:
                    mymovehub.run_motor_for_time(MOTOR_B, 300, self.right_dc)
                if self.head_dc != 0:
                    mymovehub.run_motor_for_time(MOTOR_D, 100, self.head_dc)
                if self.left_dc != 0 or self.right_dc != 0 or self.head_dc != 0:
                    sleep(0.2)

# SensorsThread is always running
# sensors getting (color and distance)
class SensorsThread(threading.Thread):

    def __init__(self):
        self.running = True
        threading.Thread.__init__(self)
        print('Distance Thread Ready')

    def run(self):
        global dangersaid
        while self.running:
            # get sensor data
            distance = mymovehub.last_distance_D
            color = mymovehub.last_color_D

            # handle distance
            if distance != '' and distance > 9:
               dangersaid = False
                    
            if distance != '' and distance < 7:                    
                MotorsThread.left_dc = 0
                MotorsThread.right_dc = 0
                if dangersaid == False:
                    espeak.synth("Stop! Danger!")
                    dangersaid = True

            if color != '':
                if colorsaid == False:
                    colorsaid = True
                    if color == 'RED':
                        espeak.synth("Red detected!")
                    if color == 'YELLOW':
                        espeak.synth("Yellow detected!")
                    if color == 'GREEN':
                        espeak.synth("Green detected!")
                    if color == 'WHITE':
                        espeak.synth("White detected!")
                    if color == 'BLACK':
                        espeak.synth("Black detected!")
                    if color == 'BLUE':
                        espeak.synth("Blue detected!")
                            
            if color == '': 
                colorsaid = False

            sleep(0.1)

# Connect to BOOST Move Hub
mymovehub = MoveHub(MY_MOVEHUB_ADD, 'Auto', 'hci0')

try:
    mymovehub.start()
    mymovehub.subscribe_all()
    mymovehub.listen_colordist_sensor(PORT_D)
    mymovehub.listen_button()

    sensors_thread = SensorsThread()
    sensors_thread.setDaemon(True)
    sensors_thread.start()

    motors_thread = MotorsThread()
    motors_thread.setDaemon(True)
    motors_thread.start()

    espeak.synth("Hello, I am Vernie. Welcome to Python!")

    while workingloop == True:
        events = get_gamepad()
        for event in events:
            if str(event.ev_type) == "Absolute" or str(event.ev_type) == "Key":
                if str(event.code) == "BTN_TRIGGER":
                    # RJ:forward
                    direction = "forward"
                    if MotorsThread.right_dc != speed or MotorsThread.left_dc != speed:
                        MotorsThread.right_dc = speed
                        MotorsThread.left_dc = speed                    

                if str(event.code) == "BTN_THUMB2":
                    # RJ:back
                    direction = "back"
                    if MotorsThread.right_dc != -speed or MotorsThread.left_dc != -speed:
                        MotorsThread.right_dc = -speed
                        MotorsThread.left_dc = -speed            

                if str(event.code) == "ABS_X" and event.state == 0:
                    # LJ:left
                    if direction == "forward":
                        MotorsThread.left_dc = 0
                        MotorsThread.right_dc = speed
                        sleep(0.3)                        
                        MotorsThread.left_dc = speed
                    if direction == "back":
                        MotorsThread.left_dc = -speed
                        MotorsThread.right_dc = 0
                        sleep(0.3)                        
                        MotorsThread.right_dc = -speed
                        
                if str(event.code) == "ABS_X" and event.state == 255:
                    # LJ:right
                    if direction == "forward":
                        MotorsThread.left_dc = speed
                        MotorsThread.right_dc = 0
                        sleep(0.3)                        
                        MotorsThread.right_dc = speed
                    if direction == "back":
                        MotorsThread.left_dc = 0
                        MotorsThread.right_dc = -speed
                        sleep(0.3)                        
                        MotorsThread.left_dc = -speed

                if str(event.code) == "ABS_Y":
                    # LJ:down
                    MotorsThread.left_dc = 0
                    MotorsThread.right_dc = 0

                if str(event.code) == "BTN_BASE3" or str(event.code) == "BTN_BASE4":
                    # LJ:down
                    espeak.synth("Stopping now!")
                    espeak.synth("Goodbye!")
                    MotorsThread.left_dc = 0
                    MotorsThread.right_dc = 0
                    workingloop = False

finally:
    motors_thread.running = False
    sensors_thread.running = False
    mymovehub.stop()
